%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% DemoShellCrossSection
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Overview:
%   Demonstrate the shift in plasmonic resonance due to changes
%   in particle size
%
% References:
%   Section 7.4.1
%   Mathematical and Computational Methods in Photonics and Phononics
%
% Authors:
%   Habib Ammari, Brian Fitzpatrick, Hyeonbae Kang,
%   Matias Ruiz, Sanghyeon Yu, Hai Zhang
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;
close all;
clc;

N_wlength = 150;L1 = 300;L2= 500;
lambda = L1 : (L2-L1)/N_wlength : L2; % values for wavelengths (unit: nm)

w = 1220./lambda; % converting wavelength(unit: nm) to frequency(unit: eV)
wp = 5.8; % plasma frequency
gamma = 0.2; % damping parameter
eps = 1-wp^2./(w.*(w+1i*gamma)); % permittivity 'epsilon' using Drude's model

epsm=1; % background permittivity
mum=1; % background permeability

r_data = [5, 10, 20, 30];

for ii=1:size(r_data,2)
    r=r_data(ii);
    r2=r/2;
    Q=zeros(1,size(w,2));
    for pp=1:size(w,2)
        
        km=2*pi/lambda(pp); % background wavenumber
        
        sumQ=0;
        N = floor((km*r)+4*(km*r)^(1/3)+2);
        for n=1:N
            
            % Scattering Coefficients
            %[W_TE_n, W_TM_n] = SCoeff_sphere(w(pp),1,eps(pp),1,1,r,n);
            [W_TE_n, W_TM_n] = SCoeff_shell3(w(pp),mum,epsm,1,eps(pp),r,r2,n);
            
            an=1i/(n*(n+1))*km*W_TE_n;
            bn=1i/(n*(n+1))*km*W_TM_n;
            
            % Scattering cross section
            %sumQ=sumQ+2/(km*r)^2*(2*n+1)*(abs(an)^2+abs(bn)^2);
            
            % Extinction cross section
            sumQ=sumQ-2/(km*r)^2*(2*n+1)*real((an)+(bn)); %
        end
        Q(pp) =  sumQ;
    end
    %
    figure(3)
    plot(lambda,Q,'linewidth', 1.5)
    hold on
    ylim([0 22])
    xlabel('Wavelength (nm)')
    ylabel('extinction cross section Q^{ext}')
    set(gca,'fontsize', 17);
    
end