%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% SCoeff_shell3
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% References:
%   Mathematical and Computational Methods in Photonics and Phononics
%
% Authors:
%   Habib Ammari, Brian Fitzpatrick, Hyeonbae Kang,
%   Matias Ruiz, Sanghyeon Yu, Hai Zhang
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [W_TE_n, W_TM_n] = SCoeff_shell3(omega,mu0,eps0,mu1,eps1,r1,r2,n)
%% make wave numbers for each layer
k0 = omega*sqrt(mu0*eps0);
k1 = omega*sqrt(mu1*eps1);

%km = 2*pi*omega/1220;

k0 = 2*pi*omega/1220;
k1 = k0*sqrt(mu1.*eps1);

mu2=mu0;
eps2=eps0;
k2 = k0;

%% Definitions for spherical bessel function, spherical hankel function and their derivatives
sbesselj = @(n,z) sqrt(pi/2/z)*besselj(n+1/2,z); % spherical bessel
sbesselh = @(n,z) sqrt(pi/2/z)*besselh(n+1/2,1,z); % spherical hankel
Dsbesselj = @(n,z) sbesselj(n-1,z)-(n+1)/z*sbesselj(n,z); % derivative of spherical bessel
Dsbesselh = @(n,z) sbesselh(n-1,z)-(n+1)/z*sbesselh(n,z); % derivative of spherical hankel
Jcal = @(n,z) sbesselj(n,z)+z*Dsbesselj(n,z); % function \mathcal{J}_n
Hcal = @(n,z) sbesselh(n,z)+z*Dsbesselh(n,z); % function \mathcal{H}_n

matTE1A = [ sbesselj(n,k0*r1), sbesselh(n,k0*r1) ; 1/mu0*Jcal(n,k0*r1), 1/mu0*Hcal(n,k0*r1)];
matTE2A = [ sbesselj(n,k1*r1), sbesselh(n,k1*r1) ; 1/mu1*Jcal(n,k1*r1), 1/mu1*Hcal(n,k1*r1)];

matTE1B = [ sbesselj(n,k1*r2), sbesselh(n,k1*r2) ; 1/mu1*Jcal(n,k1*r2), 1/mu1*Hcal(n,k1*r2)];
matTE2B = [ sbesselj(n,k2*r2), sbesselh(n,k2*r2) ; 1/mu2*Jcal(n,k2*r2), 1/mu2*Hcal(n,k2*r2)];

vec_a = [0; 1];

cc=(matTE1A)\(matTE2A*((matTE1B)\(matTE2B*vec_a)));
a0=cc(2)/cc(1);
W_TE_n = -1i*n*(n+1)/k0(0+1)*a0;

matTM1A = [ 1/eps0*Jcal(n,k0*r1), 1/eps0*Hcal(n,k0*r1) ; sbesselj(n,k0*r1), sbesselh(n,k0*r1)];
matTM2A = [ 1/eps1*Jcal(n,k1*r1), 1/eps1*Hcal(n,k1*r1) ; sbesselj(n,k1*r1), sbesselh(n,k1*r1)];

matTM1B = [ 1/eps1*Jcal(n,k1*r2), 1/eps1*Hcal(n,k1*r2) ; sbesselj(n,k1*r2), sbesselh(n,k1*r2)];
matTM2B = [ 1/eps2*Jcal(n,k2*r2), 1/eps2*Hcal(n,k2*r2) ; sbesselj(n,k2*r2), sbesselh(n,k2*r2)];

%tbL_over_bL = - Hcal(n,k1*r2)/Jcal(n,k1*r2);
vec_b = [0; 1];

dd=(matTM1A)\(matTM2A*((matTM1B)\(matTM2B*vec_b)));
b0=dd(2)/dd(1);
W_TM_n = -1i*n*(n+1)/k0(0+1)*b0;
